@@ -29,8 +29,10 @@ usage(FILE *stream, const char *progname)
 	fprintf(stream, "\n\targuments may be placed in random order\n");
 	fprintf(stream, "\n  Options:\n");
 	fprintf(stream, "\t-D\t\tenable DNSSEC (DO bit)\n");
+#ifdef HAVE_SSL
 	fprintf(stream, "\t-T\t\ttrace from the root down to <name>\n");
 	fprintf(stream, "\t-S\t\tchase signature(s) from <name> to a know key [*]\n");
+#endif /*HAVE_SSL*/
 	fprintf(stream, "\t-V <number>\tverbosity (0-5)\n");
 	fprintf(stream, "\t-Q\t\tquiet mode (overrules -V)\n");
 	fprintf(stream, "\n");
@@ -50,9 +52,6 @@ usage(FILE *stream, const char *progname)
 	fprintf(stream, "\t\t\tused to verify any signatures in the current answer\n");
 	fprintf(stream, "\t-o <mnemonic>\tset flags to: [QR|qr][AA|aa][TC|tc][RD|rd][CD|cd][RA|ra][AD|ad]\n");
 	fprintf(stream, "\t\t\tlowercase: unset bit, uppercase: set bit\n");
-#if 0
-	fprintf(stream, "\t-O <opcode>\tset the opcode to: [query, iquery, status, notify, update]]\n");
-#endif
 	fprintf(stream, "\t-p <port>\tuse <port> as remote port number\n");
 	fprintf(stream, "\t-s\t\tshow the DS RR for each key in a packet\n");
 	fprintf(stream, "\t-u\t\tsend the query with udp (the default)\n");
@@ -117,11 +116,11 @@ main(int argc, char *argv[])
 	char		*tsig_name = NULL;
 	char		*tsig_data = NULL;
 	char 		*tsig_algorithm = NULL;
-	ldns_rr		*dnssec_key;
 	size_t		tsig_separator;
 	size_t		tsig_separator2;
 	ldns_rr		*axfr_rr;
 	ldns_status	status;
+	char *type_str;
 	
 	/* list of keys used in dnssec operations */
 	ldns_rr_list	*key_list = ldns_rr_list_new(); 
@@ -212,6 +211,7 @@ main(int argc, char *argv[])
 				}
 				PURPOSE = DRILL_TRACE;
 				break;
+#ifdef HAVE_SSL
 			case 'S':
 				if (PURPOSE == DRILL_TRACE) {
 					fprintf(stderr, "-T and -S cannot be used at the same time.\n");
@@ -219,6 +219,7 @@ main(int argc, char *argv[])
 				}
 				PURPOSE = DRILL_CHASE;
 				break;
+#endif /* HAVE_SSL */
 			case 'V':
 				verbosity = atoi(optarg);
 				break;
@@ -267,11 +268,10 @@ main(int argc, char *argv[])
 				qusevc = true;
 				break;
 			case 'k':
-				dnssec_key = read_key_file(optarg);
-				if (!dnssec_key) {
-					error("Could not parse the key file: %s", optarg);
+				status = read_key_file(optarg, key_list);
+				if (status != LDNS_STATUS_OK) {
+					error("Could not parse the key file %s: %s", optarg, ldns_get_errorstr_by_id(status));
 				}
-				ldns_rr_list_push_rr(key_list, dnssec_key);
 				qdnssec = true; /* enable that too */
 				break;
 			case 'o':
@@ -339,6 +339,7 @@ main(int argc, char *argv[])
 				PURPOSE = DRILL_REVERSE;
 				break;
 			case 'y':
+#ifdef HAVE_SSL
 				if (strchr(optarg, ':')) {
 					tsig_separator = (size_t) (strchr(optarg, ':') - optarg);
 					if (strchr(optarg + tsig_separator + 1, ':')) {
@@ -360,6 +361,11 @@ main(int argc, char *argv[])
 					tsig_name[tsig_separator] = '\0';
 					tsig_data[ tsig_separator2 - tsig_separator - 1] = '\0';
 				}
+#else
+				fprintf(stderr, "TSIG requested, but SSL is not supported\n");
+				result = EXIT_FAILURE;
+				goto exit;
+#endif /* HAVE_SSL */
 				break;
 			case 'z':
 				qrandom = false;
@@ -389,10 +395,15 @@ main(int argc, char *argv[])
 
 	/* do a secure trace when requested */
 	if (PURPOSE == DRILL_TRACE && qdnssec) {
+#ifdef HAVE_SSL
 		if (ldns_rr_list_rr_count(key_list) == 0) {
 			warning("%s", "No trusted keys were given. Will not be able to verify authenticity!");
 		}
 		PURPOSE = DRILL_SECTRACE;
+#else
+		fprintf(stderr, "ldns has not been compiled with OpenSSL support. Secure trace not available\n");
+		exit(1);
+#endif /* HAVE_SSL */
 	}
 
 	/* parse the arguments, with multiple arguments, the last argument
@@ -456,8 +467,9 @@ main(int argc, char *argv[])
 		/* no server given make a resolver from /etc/resolv.conf */
 		status = ldns_resolver_new_frm_file(&res, resolv_conf_file);
 		if (status != LDNS_STATUS_OK) {
-			warning("Could not create a resolver structure: %s",
-				   ldns_get_errorstr_by_id(status));
+			warning("Could not create a resolver structure: %s (%s)\n"
+					"Try drill @localhost if you have a resolver running on your machine.",
+				    ldns_get_errorstr_by_id(status), resolv_conf_file);
 			result = EXIT_FAILURE;
 			goto exit;
 		}
@@ -546,7 +558,6 @@ main(int argc, char *argv[])
 		case DRILL_TRACE:
 			/* do a trace from the root down */
 			if (!global_dns_root) {
-
 				init_root();
 			}
 			qname = ldns_dname_new_frm_str(name);
@@ -567,7 +578,9 @@ main(int argc, char *argv[])
 				error("%s", "making qname");
 			}
 			/* don't care about return packet */
+#ifdef HAVE_SSL
 			result = do_secure_trace(res, qname, type, clas, key_list, trace_start_name);
+#endif /* HAVE_SSL */
 			clear_root();
 			break;
 		case DRILL_CHASE:
@@ -593,6 +606,7 @@ main(int argc, char *argv[])
 				if (!ldns_pkt_answer(pkt)) {
 					mesg("No answer in packet");
 				} else {
+#ifdef HAVE_SSL
 					ldns_resolver_set_dnssec_anchors(res, ldns_rr_list_clone(key_list));
 					result = do_chase(res, qname, type,
 					                  clas, key_list, 
@@ -608,6 +622,7 @@ main(int argc, char *argv[])
 							mesg("Chase failed.");
 						}
 					}
+#endif /* HAVE_SSL */
 				}
 				ldns_pkt_free(pkt);
 			}
@@ -641,6 +656,9 @@ main(int argc, char *argv[])
 		case DRILL_REVERSE:
 			/* ipv4 or ipv6 addr? */
 			if (strchr(name, ':')) {
+				if (strchr(name, '.')) {
+					error("Syntax error: both '.' and ':' seen in address\n");
+				}
 				name2 = malloc(IP6_ARPA_MAX_LEN + 20);
 				c = 0;
 				for (i=0; i<(int)strlen(name); i++) {
@@ -762,7 +780,7 @@ main(int argc, char *argv[])
 					axfr_rr = ldns_axfr_next(res);
 					if(!axfr_rr) {
 						fprintf(stderr, "AXFR failed.\n");
-						ldns_pkt_print(stdout, 
+						ldns_pkt_print(stdout,
 							ldns_axfr_last_pkt(res));
 						goto exit;
 					}
@@ -821,13 +839,34 @@ main(int argc, char *argv[])
 					}
 
 					/* verify */
-					result = ldns_pkt_verify(pkt, type, qname, key_list, NULL, NULL);
+#ifdef HAVE_SSL
+					key_verified = ldns_rr_list_new();
+					result = ldns_pkt_verify(pkt, type, qname, key_list, NULL, key_verified);
 
-					if (result == LDNS_STATUS_OK) {
+					if (result == LDNS_STATUS_ERR) {
+						/* is the existence denied then? */
+						result = ldns_verify_denial(pkt, qname, type, NULL, NULL);
+						if (result == LDNS_STATUS_OK) {
+							if (verbosity != -1) {
+								printf("Existence denied for ");
+								ldns_rdf_print(stdout, qname);
+								type_str = ldns_rr_type2str(type);
+								printf("\t%s\n", type_str);
+								LDNS_FREE(type_str);
+							}
+						} else {
+							if (verbosity != -1) {
+								printf("Bad data; RR for name and "
+								       "type not found or failed to "
+								       "verify, and denial of "
+								       "existence failed.\n");
+							}
+						}
+					} else if (result == LDNS_STATUS_OK) {
 						for(key_count = 0; key_count < ldns_rr_list_rr_count(key_verified);
 								key_count++) {
 							if (verbosity != -1) {
-								printf("VALIDATED by id = %d, owner = ",
+								printf("; VALIDATED by id = %d, owner = ",
 										(int)ldns_calc_keytag(
 												      ldns_rr_list_rr(key_verified, key_count)));
 								ldns_rdf_print(stdout, ldns_rr_owner(
@@ -839,8 +878,9 @@ main(int argc, char *argv[])
 						for(key_count = 0; key_count < ldns_rr_list_rr_count(key_list);
 								key_count++) {
 							if (verbosity != -1) {
-								printf("BOGUS by id = %d, owner = ",
-										(int)ldns_calc_keytag(
+								printf("; %s for id = %d, owner = ",
+								       ldns_get_errorstr_by_id(result),
+								       (int)ldns_calc_keytag(
 												      ldns_rr_list_rr(key_list, key_count)));
 								ldns_rdf_print(stdout, ldns_rr_owner(
 
@@ -849,9 +889,11 @@ main(int argc, char *argv[])
 								printf("\n");
 							}
 						}
-
 					}
-
+					ldns_rr_list_free(key_verified);
+#else
+					(void) key_count;
+#endif /* HAVE_SSL */
 				}
 				if (answer_file) {
 					dump_hex(pkt, answer_file);
