@@ -18,6 +18,10 @@
 #endif
 #include <errno.h>
 
+#ifndef IP_OFFMASK
+#define IP_OFFMASK 0x1fff
+#endif
+
 int verbosity = 1;
 
 #define ETHER_HEADER_LENGTH 14
@@ -146,7 +150,7 @@ const ldns_lookup_table lt_operators[] = {
 	{ 0, NULL }
 };
 
-const char *get_op_str(type_operator op) {
+static const char *get_op_str(type_operator op) {
 	const ldns_lookup_table *lt;
 	lt = ldns_lookup_by_id((ldns_lookup_table *) lt_operators, op);
 	if (lt) {
@@ -157,7 +161,7 @@ const char *get_op_str(type_operator op) {
 	}
 }
 
-type_operator
+static type_operator
 get_op_id(char *op_str)
 {
 	const ldns_lookup_table *lt;
@@ -291,7 +295,7 @@ get_match_by_name(char *name) {
 	return NULL;
 }
 
-match_table *
+static match_table *
 get_match_by_id(match_id id) {
 	match_table *mt = (match_table *) matches;
 
@@ -304,7 +308,7 @@ get_match_by_id(match_id id) {
 	return NULL;
 }
 
-const char *
+static const char *
 get_match_name_str(match_id id) {
 	match_table *mt = get_match_by_id(id);
 	if (mt) {
@@ -316,16 +320,7 @@ get_match_name_str(match_id id) {
 	}
 }
 
-bool is_match_name(char *name) {
-	match_table *mt = get_match_by_name(name);
-	if (mt) {
-		return true;
-	} else {
-		return false;
-	}
-}
-
-void
+static void
 print_match_operation(FILE *output, match_operation *mc)
 {
 	match_table *mt = NULL;
@@ -402,7 +397,7 @@ print_match_operation(FILE *output, match_operation *mc)
 	}
 }
 
-void
+static void
 print_match_expression(FILE *output, match_expression *expr)
 {
 	if (expr) {
@@ -448,7 +443,7 @@ fprintf(output, "_");
 	}			
 }
 
-size_t 
+static size_t 
 calculate_counters_total(match_counters *counters)
 {
 	size_t result = 0;
@@ -467,7 +462,7 @@ calculate_counters_total(match_counters *counters)
 	return result;
 }
 
-void
+static void
 print_counters(FILE *output, match_counters *counters, bool show_percentages, size_t total, int count_minimum)
 {
 	double percentage;
@@ -497,7 +492,7 @@ print_counters(FILE *output, match_counters *counters, bool show_percentages, si
 	return;	
 }
 
-void
+static void
 ldns_pkt2file_hex(FILE *fp, const ldns_pkt *pkt)
 {
 	uint8_t *wire;
@@ -535,7 +530,7 @@ ldns_pkt2file_hex(FILE *fp, const ldns_pkt *pkt)
  * (if they are 'under' this one in the tree, which should be the case in
  * the unique counter tree
  */
-size_t
+static size_t
 calculate_total_value(match_counters *counters, match_operation *cur)
 {
 	size_t result = 0;
@@ -558,7 +553,7 @@ calculate_total_value(match_counters *counters, match_operation *cur)
 	return result;
 }
 
-size_t
+static size_t
 calculate_total_count_matches(match_counters *counters, match_operation *cur)
 {
 	size_t result = 0;
@@ -595,7 +590,7 @@ calculate_total_count_matches(match_counters *counters, match_operation *cur)
  * Returns true if there is a previous match operation with the given type
  * in the counters structure
  */
-bool
+static bool
 has_previous_match(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -622,7 +617,7 @@ has_previous_match(match_counters *counters, match_operation *cur)
  * Returns true if there is a later match operation with the given type
  * in the counters structure
  */
-bool
+static bool
 has_next_match(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -649,7 +644,7 @@ has_next_match(match_counters *counters, match_operation *cur)
  * Returns the first match with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_first_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -671,7 +666,7 @@ get_first_match_expression(match_counters *counters, match_operation *cur)
  * Returns the second match expression with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_second_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -698,7 +693,7 @@ get_second_match_expression(match_counters *counters, match_operation *cur)
  * Returns the last match expression with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_last_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -720,7 +715,7 @@ get_last_match_expression(match_counters *counters, match_operation *cur)
  * Returns the last but one match expression with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_last_but_one_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -743,7 +738,7 @@ get_last_but_one_match_expression(match_counters *counters, match_operation *cur
 	}
 }
 
-size_t
+static size_t
 get_first_count(match_counters *counters, match_operation *cur)
 {
 	match_expression *o = get_first_match_expression(counters, cur);
@@ -754,7 +749,7 @@ get_first_count(match_counters *counters, match_operation *cur)
 	}
 }
 
-size_t
+static size_t
 get_last_count(match_counters *counters, match_operation *cur)
 {
 	match_expression *o = get_last_match_expression(counters, cur);
@@ -766,7 +761,7 @@ get_last_count(match_counters *counters, match_operation *cur)
 }
 
 
-size_t
+static size_t
 calculate_total_count(match_counters *counters, match_operation *cur)
 {
 	size_t result = 0;
@@ -789,7 +784,7 @@ calculate_total_count(match_counters *counters, match_operation *cur)
 	return result;
 }
 
-void
+static void
 print_counter_averages(FILE *output, match_counters *counters, match_operation *cur)
 {
 	size_t total_value;
@@ -828,7 +823,7 @@ print_counter_averages(FILE *output, match_counters *counters, match_operation *
 	return;	
 }
 
-void
+static void
 print_counter_average_count(FILE *output, match_counters *counters, match_operation *cur, bool remove_first_last)
 {
 	size_t total_matches;
@@ -887,7 +882,7 @@ print_counter_average_count(FILE *output, match_counters *counters, match_operat
 	return;	
 }
 
-bool
+static bool
 match_int(type_operator operator,
           char *value,
 	  char *mvalue)
@@ -926,10 +921,11 @@ match_int(type_operator operator,
 	}
 }
 
-bool
+static bool
 match_opcode(type_operator operator,
              char *value,
-             char *mvalue) {
+             char *mvalue)
+{
 	ldns_pkt_opcode a, b;
 	int i;
 	ldns_lookup_table *lt;
@@ -991,7 +987,7 @@ match_opcode(type_operator operator,
 	}
 }
 
-bool
+static bool
 match_str(type_operator operator,
           char *value,
           char *mvalue)
@@ -1023,7 +1019,7 @@ match_str(type_operator operator,
 	}	
 }
 
-bool
+static bool
 match_rr_type(type_operator operator,
               char *value,
 	      char *mvalue)
@@ -1058,10 +1054,11 @@ match_rr_type(type_operator operator,
 	}
 }
 
-bool
+static bool
 match_rcode(type_operator operator,
              char *value,
-             char *mvalue) {
+             char *mvalue)
+{
 	int a, b;
 	int i;
 	ldns_lookup_table *lt;
@@ -1124,7 +1121,7 @@ match_rcode(type_operator operator,
 	}
 }
 
-bool
+static bool
 value_matches(match_id id,
         type_operator operator,
         char *value,
@@ -1187,7 +1184,7 @@ value_matches(match_id id,
 	return result;
 }
 
-char *
+static char *
 get_string_value(match_id id, ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr)
 {
 	char *val;
@@ -1327,30 +1324,7 @@ get_string_value(match_id id, ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_a
 	return val;
 }
 
-/*
-bool
-match_pkt(ldns_pkt *pkt, match_counter *counter)
-{
-	bool result;
-	char *val;
-
-	if (!pkt || !counter) {
-		return false;
-	} else {
-		val = get_string_value(counter->id, pkt);
-		if (!val) {
-			return false;
-		}
-		result = value_matches(counter->id, counter->operator, val, counter->value);
-		if (result) {
-			counter->count++;
-		}
-		free(val);
-		return result;
-	}
-}
-*/
-bool
+static bool
 match_packet_to_operation(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_operation *operation)
 {
 	bool result;
@@ -1369,61 +1343,7 @@ match_packet_to_operation(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr,
 	}
 }
 
-
-
-/*
-int
-add_counter(match_counters *counters,
-            match_expression *expr)
-{
-		counters->counter = realloc(counters->counter, (counters->size + 1) * sizeof(match_expression *));
-		counters->counter[counters->size] = expr;
-		counters->size = counters->size + 1;
-
-		return 0;
-}
-*/
-
-int
-match_expression_compare_count(const void *a, const void *b)
-{
-	match_expression *mea, *meb;
-	
-	if (!a) {
-		return 1;
-	} else if (!b) {
-		return -1;
-	} else {
-		mea = (match_expression *) a;
-		meb = (match_expression *) b;
-		
-		if (mea->count < meb->count) {
-			return -1;
-		} else if (mea->count > meb->count) {
-			return 1;
-		} else {
-			return 0;
-		}
-	}
-}
-
-int
-match_expression_compare_count_p(const void *a, const void *b)
-{
-	match_expression **pmea, **pmeb;
-	
-	if (!a) {
-		return 1;
-	} else if (!b) {
-		return -1;
-	} else {
-		pmea = (match_expression **) a;
-		pmeb = (match_expression **) b;
-		return match_expression_compare_count(*pmea, *pmeb);
-	}
-}
-
-int
+static int
 match_operation_compare(const void *a, const void *b)
 {
 	match_operation *moa, *mob;
@@ -1475,7 +1395,7 @@ match_operation_compare(const void *a, const void *b)
 	}
 }
 
-int
+static int
 match_expression_compare(const void *a, const void *b)
 {
 	match_expression *mea, *meb;
@@ -1514,27 +1434,11 @@ match_expression_compare(const void *a, const void *b)
 		}
 	}
 }
-int
-match_expression_compare_p(const void *a, const void *b)
-{
-	
-	match_expression **pmea, **pmeb;
-	
-	if (!a) {
-		return 1;
-	} else if (!b) {
-		return -1;
-	} else {
-		pmea = (match_expression **) a;
-		pmeb = (match_expression **) b;
-		return match_expression_compare(*pmea, *pmeb);
-	}
-}
 
 /**
  * If count is true, and the counter is found, its count is increased by 1
  */
-int
+static int
 add_match_counter(match_counters *counters,
 		  match_expression *expr,
                   bool count)
@@ -1589,7 +1493,7 @@ add_match_counter(match_counters *counters,
 	return 0;
 }
 
-bool
+static bool
 match_dns_packet_to_expr(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_expression *expr)
 {
 	bool result;
@@ -1630,7 +1534,7 @@ match_dns_packet_to_expr(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr,
 	return result;
 }
 
-bool
+static bool
 match_expression_equals(match_expression *expr1, match_expression *expr2)
 {
 	if (!expr1 || !expr2) {
@@ -1661,8 +1565,7 @@ match_expression_equals(match_expression *expr1, match_expression *expr2)
 
 }
 
-
-void
+static void
 free_match_operation(match_operation *operation)
 {
 	if (operation) {
@@ -1673,7 +1576,7 @@ free_match_operation(match_operation *operation)
 	}
 }
 
-void
+static void
 free_match_expression(match_expression *expr)
 {
 	if (expr) {
@@ -1691,7 +1594,7 @@ free_match_expression(match_expression *expr)
 	}
 }
 
-void
+static void
 free_counters(match_counters *counters)
 {
 	if (counters) {
@@ -1708,7 +1611,7 @@ free_counters(match_counters *counters)
 	}
 }
 
-void
+static void
 match_pkt_counters(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_counters *counts)
 {
 	if (counts->left) {
@@ -1726,7 +1629,7 @@ match_pkt_counters(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_
 	}	
 }
 
-void
+static void
 match_pkt_uniques(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_counters *uniques, match_id unique_ids[], size_t unique_id_count)
 {
 	match_expression *me;
@@ -1789,26 +1692,7 @@ match_pkt_uniques(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_c
 #endif
 }
 
-
-/*
-void
-print_match_counter(FILE *output, match_counter *mc)
-{
-	match_table *mt = NULL;
-	if (mc) {
-		mt = get_match_by_id(mc->id);
-		if (mt) {
-			fprintf(output, "%s %s '%s'", mt->name, get_op_str(mc->operator), mc->value);
-		} else {
-			fprintf(output, "%u %s '%s'", mc->id, get_op_str(mc->operator), mc->value);
-		}
-	} else {
-		fprintf(output, "(nil)");
-	}
-}
-
-*/
-match_expression *
+static match_expression *
 parse_match_expression(char *string)
 {
 	match_expression *expr;
@@ -1826,21 +1710,13 @@ parse_match_expression(char *string)
 
 	j = 0;
 	for (i = 0; i < strlen(string); i++) {
-/*
 		if(!isspace(string[i])) {
-*/
 			str[j] = string[i];
 			j++;
-/*
 		}
-*/
 	}
 	str[j] = '\0';
 	
-	/*
-	printf("Parsing: %s\n", string);
-	printf("Parsing short : %s\n", str);
-	*/
 	expr = malloc(sizeof(match_expression));
 	expr->left = NULL;
 	expr->right = NULL;
@@ -2213,7 +2089,7 @@ handle_ether_packet(const u_char *data, struct pcap_pkthdr cur_hdr, match_counte
 	struct ip *iptr;
 	struct ip6_hdr *ip6_hdr;
 	int ip_hdr_size;
-	u_int8_t protocol;
+	uint8_t protocol;
 	size_t data_offset = 0;
 	ldns_rdf *src_addr, *dst_addr;
 	uint8_t *ap;
@@ -2358,7 +2234,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 			}
 
 			ip_hdr_size = (int) iptr->ip_hl * 4;
-			protocol = (u_int8_t) iptr->ip_p;
+			protocol = (uint8_t) iptr->ip_p;
 			
 			data_offset += ip_hdr_size;
 
@@ -2499,7 +2375,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 		}
 
 		ip_hdr_size = IP6_HEADER_LENGTH;
-		protocol = (u_int8_t) ip6_hdr->ip6_ctlun.ip6_un1.ip6_un1_nxt;
+		protocol = (uint8_t) ip6_hdr->ip6_ctlun.ip6_un1.ip6_un1_nxt;
 		
 		data_offset += ip_hdr_size;
 
@@ -2586,7 +2462,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 			/* tcp packets are skipped */
 			tcp_packets++;
 		} else {
-			printf("ipv6 unknown next header type: %u\n", protocol);
+			printf("ipv6 unknown next header type: %u\n", (unsigned int) protocol);
 		}
 
 
