@@ -27,6 +27,9 @@ ldns_lookup_table ldns_signing_algorithms[] = {
 #ifdef USE_SHA2
         { LDNS_SIGN_RSASHA256, "RSASHA256" },
         { LDNS_SIGN_RSASHA512, "RSASHA512" },
+#endif
+#ifdef USE_GOST
+        { LDNS_SIGN_GOST, "GOST" },
 #endif
         { LDNS_SIGN_DSA, "DSA" },
         { LDNS_SIGN_DSA_NSEC3, "DSA_NSEC3" },
@@ -73,7 +76,6 @@ ldns_key_new()
 		ldns_key_set_external_key(newkey, NULL);
 		return newkey;
 	}
-	return NULL;
 }
 
 ldns_status
@@ -100,6 +102,85 @@ ldns_key_new_frm_engine(ldns_key **key, ENGINE *e, char *key_id, ldns_algorithm
 }
 #endif
 
+#ifdef USE_GOST
+int
+ldns_key_EVP_load_gost_id(void)
+{
+	static int gost_id = 0;
+	const EVP_PKEY_ASN1_METHOD* meth;
+	ENGINE* e;
+
+	if(gost_id) return gost_id;
+
+	/* see if configuration loaded gost implementation from other engine*/
+	meth = EVP_PKEY_asn1_find_str(NULL, "gost2001", -1);
+	if(meth) {
+		EVP_PKEY_asn1_get0_info(&gost_id, NULL, NULL, NULL, NULL, meth);
+		return gost_id;
+	}
+
+	/* see if engine can be loaded already */
+	e = ENGINE_by_id("gost");
+	if(!e) {
+		/* load it ourself, in case statically linked */
+		ENGINE_load_builtin_engines();
+		ENGINE_load_dynamic();
+		e = ENGINE_by_id("gost");
+	}
+	if(!e) {
+		/* no gost engine in openssl */
+		return 0;
+	}
+	if(!ENGINE_set_default(e, ENGINE_METHOD_ALL)) {
+		ENGINE_finish(e);
+		ENGINE_free(e);
+		return 0;
+	}
+
+	meth = EVP_PKEY_asn1_find_str(&e, "gost2001", -1);
+	ENGINE_finish(e);
+	ENGINE_free(e);
+	if(!meth) {
+		/* algo not found */
+		return 0;
+	}
+	
+	EVP_PKEY_asn1_get0_info(&gost_id, NULL, NULL, NULL, NULL, meth);
+	return gost_id;
+}
+
+/** read GOST private key */
+static EVP_PKEY*
+ldns_key_new_frm_fp_gost_l(FILE* fp, int* line_nr)
+{
+	char token[16384];
+	const unsigned char* pp;
+	int gost_id;
+	EVP_PKEY* pkey;
+	ldns_rdf* b64rdf = NULL;
+
+	gost_id = ldns_key_EVP_load_gost_id();
+	if(!gost_id)
+		return NULL;
+
+	if (ldns_fget_keyword_data_l(fp, "GostAsn1", ": ", token, "\n", 
+		sizeof(token), line_nr) == -1)
+		return NULL;
+	while(strlen(token) < 96) {
+		/* read more b64 from the file, b64 split on multiple lines */
+		if(ldns_fget_token_l(fp, token+strlen(token), "\n",
+			sizeof(token)-strlen(token), line_nr) == -1)
+			return NULL;
+	}
+	if(ldns_str2rdf_b64(&b64rdf, token) != LDNS_STATUS_OK)
+		return NULL;
+	pp = (unsigned char*)ldns_rdf_data(b64rdf);
+	pkey = d2i_PrivateKey(gost_id, NULL, &pp, ldns_rdf_size(b64rdf));
+	ldns_rdf_deep_free(b64rdf);
+	return pkey;
+}
+#endif
+
 ldns_status
 ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 {
@@ -151,22 +232,22 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		alg = LDNS_SIGN_RSAMD5;
 	}
 	if (strncmp(d, "2 DH", 2) == 0) {
-		alg = LDNS_DH;
+		alg = (ldns_signing_algorithm)LDNS_DH;
 	}
 	if (strncmp(d, "3 DSA", 2) == 0) {
 		alg = LDNS_SIGN_DSA;
 	}
 	if (strncmp(d, "4 ECC", 2) == 0) {
-		alg = LDNS_ECC;
+		alg = (ldns_signing_algorithm)LDNS_ECC;
 	}
 	if (strncmp(d, "5 RSASHA1", 2) == 0) {
 		alg = LDNS_SIGN_RSASHA1;
 	}
 	if (strncmp(d, "6 DSA", 2) == 0) {
-		alg = LDNS_DSA_NSEC3;
+		alg = LDNS_SIGN_DSA_NSEC3;
 	}
 	if (strncmp(d, "7 RSASHA1", 2) == 0) {
-		alg = LDNS_RSASHA1_NSEC3;
+		alg = LDNS_SIGN_RSASHA1_NSEC3;
 	}
 
 	if (strncmp(d, "8 RSASHA256", 2) == 0) {
@@ -183,6 +264,14 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 #else
 		fprintf(stderr, "Warning: SHA512 not compiled into this ");
 		fprintf(stderr, "version of ldns\n");
+#endif
+	}
+	if (strncmp(d, "249 GOST", 4) == 0) {
+#ifdef USE_GOST
+		alg = LDNS_SIGN_GOST;
+#else
+		fprintf(stderr, "Warning: GOST not compiled into this ");
+		fprintf(stderr, "version of ldns\n");
 #endif
 	}
 	if (strncmp(d, "157 HMAC-MD5", 4) == 0) {
@@ -208,6 +297,10 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 			ldns_key_set_algorithm(k, alg);
 #ifdef HAVE_SSL
 			rsa = ldns_key_new_frm_fp_rsa_l(fp, line_nr);
+			if (!rsa) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
 			ldns_key_set_rsa_key(k, rsa);
 			RSA_free(rsa);
 #endif /* HAVE_SSL */
@@ -217,6 +310,10 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 			ldns_key_set_algorithm(k, alg);
 #ifdef HAVE_SSL
 			dsa = ldns_key_new_frm_fp_dsa_l(fp, line_nr);
+			if (!dsa) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
 			ldns_key_set_dsa_key(k, dsa);
 			DSA_free(dsa);
 #endif /* HAVE_SSL */
@@ -227,16 +324,30 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 			ldns_key_set_algorithm(k, alg);
 #ifdef HAVE_SSL
 			hmac = ldns_key_new_frm_fp_hmac_l(fp, line_nr, &hmac_size);
+			if (!hmac) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
 			ldns_key_set_hmac_size(k, hmac_size);
 			ldns_key_set_hmac_key(k, hmac);
 #endif /* HAVE_SSL */
 			break;
+		case LDNS_SIGN_GOST:
+			ldns_key_set_algorithm(k, alg);
+#if defined(HAVE_SSL) && defined(USE_GOST)
+			ldns_key_set_evp_key(k, 
+				ldns_key_new_frm_fp_gost_l(fp, line_nr));
+			if(!k->_key.key) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
+#endif
+			break;
 		case 0:
 		default:
 			return LDNS_STATUS_SYNTAX_ALG_ERR;
 			break;
 	}
-
 	key_rr = ldns_key2rr(k);
 	ldns_key_set_keytag(k, ldns_calc_keytag(key_rr));
 	ldns_rr_free(key_rr);
@@ -512,6 +623,39 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 }
 #endif /* HAVE_SSL */
 
+#ifdef USE_GOST
+static EVP_PKEY*
+ldns_gen_gost_key(void)
+{
+	EVP_PKEY_CTX* ctx;
+	EVP_PKEY* p = NULL;
+	int gost_id = ldns_key_EVP_load_gost_id();
+	if(!gost_id)
+		return NULL;
+	ctx = EVP_PKEY_CTX_new_id(gost_id, NULL);
+	if(!ctx) {
+		/* the id should be available now */
+		return NULL;
+	}
+	if(EVP_PKEY_CTX_ctrl_str(ctx, "paramset", "A") <= 0) {
+		/* cannot set paramset */
+		EVP_PKEY_CTX_free(ctx);
+		return NULL;
+	}
+
+	if(EVP_PKEY_keygen_init(ctx) <= 0) {
+		EVP_PKEY_CTX_free(ctx);
+		return NULL;
+	}
+	if(EVP_PKEY_keygen(ctx, &p) <= 0) {
+		EVP_PKEY_free(p);
+		EVP_PKEY_CTX_free(ctx);
+		return NULL;
+	}
+	EVP_PKEY_CTX_free(ctx);
+	return p;
+}
+#endif
 
 ldns_key *
 ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
@@ -589,6 +733,11 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 
 			ldns_key_set_flags(k, 0);
 			break;
+		case LDNS_SIGN_GOST:
+#if defined(HAVE_SSL) && defined(USE_GOST)
+			ldns_key_set_evp_key(k, ldns_gen_gost_key());
+#endif /* HAVE_SSL and USE_GOST */
+			break;
 	}
 	ldns_key_set_algorithm(k, alg);
 	return k;
@@ -949,6 +1098,29 @@ ldns_key_dsa2bin(unsigned char *data, DSA *k, uint16_t *size)
 	*size = 21 + (*size * 3);
 	return true;
 }
+
+#ifdef USE_GOST
+static bool
+ldns_key_gost2bin(unsigned char* data, EVP_PKEY* k, uint16_t* size)
+{
+	int i;
+	unsigned char* pp = NULL;
+	if(i2d_PUBKEY(k, &pp) != 37 + 64) {
+		/* expect 37 byte(ASN header) and 64 byte(X and Y) */
+		CRYPTO_free(pp);
+		return false;
+	}
+	/* omit ASN header */
+	/* insert parameters */
+	data[0] = 0;
+	data[1] = 0;
+	for(i=0; i<64; i++)
+		data[i+2] = pp[i+37];
+	CRYPTO_free(pp);
+	*size = 66;
+	return true;
+}
+#endif /* USE_GOST */
 #endif /* HAVE_SSL */
 
 ldns_rr *
@@ -1057,6 +1229,19 @@ ldns_key2rr(const ldns_key *k)
 			}
 #endif /* HAVE_SSL */
 			break;
+		case LDNS_SIGN_GOST:
+			ldns_rr_push_rdf(pubkey, ldns_native2rdf_int8(
+				LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
+#if defined(HAVE_SSL) && defined(USE_GOST)
+			bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+			if (!bin) 
+				return NULL;
+			if (!ldns_key_gost2bin(bin, k->_key.key, &size)) {
+				return NULL;
+			}
+			internal_data = 1;
+			break;
+#endif /* HAVE_SSL and USE_GOST */
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
@@ -1176,3 +1361,13 @@ ldns_key_get_file_base_name(ldns_key *key)
 	return file_base_name;
 }
 
+int ldns_key_algo_supported(int algo)
+{
+	ldns_lookup_table *lt = ldns_signing_algorithms;
+	while(lt->name) {
+		if(lt->id == algo)
+			return 1;
+		lt++;
+	}
+	return 0;
+}
