@@ -29,6 +29,7 @@
 #endif
 #include <sys/time.h>
 #include <errno.h>
+#include <fcntl.h>
 
 ldns_status
 ldns_send(ldns_pkt **result_packet, ldns_resolver *r, const ldns_pkt *query_pkt)
@@ -218,6 +219,67 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 	return status;
 }
 
+/** best effort to set nonblocking */
+static void
+ldns_sock_nonblock(int sockfd)
+{
+#ifdef HAVE_FCNTL
+	int flag;
+	if((flag = fcntl(sockfd, F_GETFL)) != -1) {
+		flag |= O_NONBLOCK;
+		if(fcntl(sockfd, F_SETFL, flag) == -1) {
+			/* ignore error, continue blockingly */
+		}
+	}
+#elif defined(HAVE_IOCTLSOCKET)
+	unsigned long on = 1;
+	if(ioctlsocket(sockfd, FIONBIO, &on) != 0) {
+		/* ignore error, continue blockingly */
+	}
+#endif
+}
+
+/** best effort to set blocking */
+static void
+ldns_sock_block(int sockfd)
+{
+#ifdef HAVE_FCNTL
+	int flag;
+	if((flag = fcntl(sockfd, F_GETFL)) != -1) {
+		flag &= ~O_NONBLOCK;
+		if(fcntl(sockfd, F_SETFL, flag) == -1) {
+			/* ignore error, continue */
+		}
+	}
+#elif defined(HAVE_IOCTLSOCKET)
+	unsigned long off = 0;
+	if(ioctlsocket(sockfd, FIONBIO, &off) != 0) {
+		/* ignore error, continue */
+	}
+#endif
+}
+
+/** wait for a socket to become ready */
+static int
+ldns_sock_wait(int sockfd, struct timeval timeout, int write)
+{
+	fd_set fds;
+	int ret;
+	FD_ZERO(&fds);
+	FD_SET(FD_SET_T sockfd, &fds);
+	if(write)
+		ret = select(sockfd+1, NULL, &fds, NULL, &timeout);
+	else
+		ret = select(sockfd+1, &fds, NULL, NULL, &timeout);
+	if(ret == 0)
+		/* timeout expired */
+		return 0;
+	else if(ret == -1)
+		/* error */
+		return 0;
+	return 1;
+}
+
 ldns_status
 ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage *to, 
 		socklen_t tolen, struct timeval timeout, size_t *answer_size)
@@ -232,6 +294,11 @@ ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage
 	}
 
 	/* wait for an response*/
+	if(!ldns_sock_wait(sockfd, timeout, 0)) {
+		close(sockfd);
+		return LDNS_STATUS_NETWORK_ERR;
+	}
+	
 	answer = ldns_udp_read_wire(sockfd, answer_size, NULL, NULL);
 	close(sockfd);
 
@@ -263,7 +330,7 @@ ldns_udp_bgsend(ldns_buffer *qbin, const struct sockaddr_storage *to, socklen_t
 }
 
 int
-ldns_udp_connect(const struct sockaddr_storage *to, struct timeval timeout)
+ldns_udp_connect(const struct sockaddr_storage *to, struct timeval ATTR_UNUSED(timeout))
 {
 	int sockfd;
 	
@@ -272,14 +339,6 @@ ldns_udp_connect(const struct sockaddr_storage *to, struct timeval timeout)
 			== -1) {
                 return 0;
         }
-	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
-				(socklen_t)sizeof(timeout))) {
-                /* might fail, in that case, use default for now */
-		/*
-		close(sockfd);
-		return 0;
-		*/
-        }
 	return sockfd;
 }
 
@@ -294,22 +353,76 @@ ldns_tcp_connect(const struct sockaddr_storage *to, socklen_t tolen,
 		return 0;
 	}
 
-	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
-				(socklen_t) sizeof(timeout))) {
-		close(sockfd);
-		return 0;
-	}
-	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (void*)&timeout,
-				(socklen_t) sizeof(timeout))) {
-		close(sockfd);
-		return 0;
+	/* perform nonblocking connect, to be able to wait with select() */
+	ldns_sock_nonblock(sockfd);
+	if (connect(sockfd, (struct sockaddr*)to, tolen) == -1) {
+#ifndef USE_WINSOCK
+#ifdef EINPROGRESS
+		if(errno != EINPROGRESS) {
+#else
+		if(1) {
+#endif
+			close(sockfd);
+			return 0;
+		}
+#else /* USE_WINSOCK */
+		if(WSAGetLastError() != WSAEINPROGRESS &&
+			WSAGetLastError() != WSAEWOULDBLOCK) {
+			closesocket(sockfd);
+			return 0;
+		}
+#endif
+		/* error was only telling us that it would block */
 	}
 
-	if (connect(sockfd, (struct sockaddr*)to, tolen) == -1) {
- 		close(sockfd);
-		return 0;
+	/* wait(write) until connected or error */
+	while(1) {
+		int error = 0;
+		socklen_t len = (socklen_t)sizeof(error);
+
+		if(!ldns_sock_wait(sockfd, timeout, 1)) {
+			close(sockfd);
+			return 0;
+		}
+
+		/* check if there is a pending error for nonblocking connect */
+		if(getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (void*)&error,
+			&len) < 0) {
+#ifndef USE_WINSOCK
+			error = errno; /* on solaris errno is error */
+#else
+			error = WSAGetLastError();
+#endif
+		}
+#ifndef USE_WINSOCK
+#if defined(EINPROGRESS) && defined(EWOULDBLOCK)
+		if(error == EINPROGRESS || error == EWOULDBLOCK)
+			continue; /* try again */
+#endif
+		else if(error != 0) {
+			close(sockfd);
+			/* error in errno for our user */
+			errno = error;
+			return 0;
+		}
+#else /* USE_WINSOCK */
+		if(error == WSAEINPROGRESS)
+			continue;
+		else if(error == WSAEWOULDBLOCK)
+			continue;
+		else if(error != 0) {
+			closesocket(sockfd);
+			errno = error;
+			return 0;
+		}
+#endif /* USE_WINSOCK */
+		/* connected */
+		break;
 	}
 
+	/* set the socket blocking again */
+	ldns_sock_block(sockfd);
+
 	return sockfd;
 }
 
@@ -325,7 +438,7 @@ ldns_tcp_send_query(ldns_buffer *qbin, int sockfd,
 	ldns_write_uint16(sendbuf, ldns_buffer_position(qbin));
 	memcpy(sendbuf + 2, ldns_buffer_export(qbin), ldns_buffer_position(qbin));
 
-	bytes = sendto(sockfd, sendbuf,
+	bytes = sendto(sockfd, (void*)sendbuf,
 			ldns_buffer_position(qbin) + 2, 0, (struct sockaddr *)to, tolen);
 
         LDNS_FREE(sendbuf);
@@ -343,7 +456,7 @@ ldns_udp_send_query(ldns_buffer *qbin, int sockfd, const struct sockaddr_storage
 {
 	ssize_t bytes;
 
-	bytes = sendto(sockfd, ldns_buffer_begin(qbin),
+	bytes = sendto(sockfd, (void*)ldns_buffer_begin(qbin),
 			ldns_buffer_position(qbin), 0, (struct sockaddr *)to, tolen);
 
 	if (bytes == -1 || (size_t)bytes != ldns_buffer_position(qbin)) {
@@ -361,7 +474,6 @@ ldns_udp_read_wire(int sockfd, size_t *size, struct sockaddr_storage *from,
 {
 	uint8_t *wire;
 	ssize_t wire_size;
-	socklen_t flen;
 
 	wire = LDNS_XMALLOC(uint8_t, LDNS_MAX_PACKETLEN);
 	if (!wire) {
@@ -369,14 +481,8 @@ ldns_udp_read_wire(int sockfd, size_t *size, struct sockaddr_storage *from,
 		return NULL;
 	}
 
-	wire_size = recvfrom(sockfd, wire, LDNS_MAX_PACKETLEN, 0, 
-			(struct sockaddr*) from, &flen);
-
-	if (from) {
-		if (fromlen) {
-			*fromlen = flen;
-		}
-	}
+	wire_size = recvfrom(sockfd, (void*)wire, LDNS_MAX_PACKETLEN, 0, 
+			(struct sockaddr *)from, fromlen);
 
 	/* recvfrom can also return 0 */
 	if (wire_size == -1 || wire_size == 0) {
@@ -391,6 +497,58 @@ ldns_udp_read_wire(int sockfd, size_t *size, struct sockaddr_storage *from,
 	return wire;
 }
 
+uint8_t *
+ldns_tcp_read_wire_timeout(int sockfd, size_t *size, struct timeval timeout)
+{
+	uint8_t *wire;
+	uint16_t wire_size;
+	ssize_t bytes = 0;
+
+	wire = LDNS_XMALLOC(uint8_t, 2);
+	if (!wire) {
+		*size = 0;
+		return NULL;
+	}
+	
+	while (bytes < 2) {
+		if(!ldns_sock_wait(sockfd, timeout, 0)) {
+			*size = 0;
+			LDNS_FREE(wire);
+			return NULL;
+		}
+		bytes = recv(sockfd, (void*)wire, 2, 0);
+		if (bytes == -1 || bytes == 0) {
+			*size = 0;
+			LDNS_FREE(wire);
+			return NULL;
+		}
+	}
+
+	wire_size = ldns_read_uint16(wire);
+	
+	LDNS_FREE(wire);
+	wire = LDNS_XMALLOC(uint8_t, wire_size);
+	bytes = 0;
+
+	while (bytes < (ssize_t) wire_size) {
+		if(!ldns_sock_wait(sockfd, timeout, 0)) {
+			*size = 0;
+			LDNS_FREE(wire);
+			return NULL;
+		}
+		bytes += recv(sockfd, (void*) (wire + bytes), 
+				(size_t) (wire_size - bytes), 0);
+		if (bytes == -1 || bytes == 0) {
+			LDNS_FREE(wire);
+			*size = 0;
+			return NULL;
+		}
+	}
+	
+	*size = (size_t) bytes;
+	return wire;
+}
+
 uint8_t *
 ldns_tcp_read_wire(int sockfd, size_t *size)
 {
@@ -405,7 +563,7 @@ ldns_tcp_read_wire(int sockfd, size_t *size)
 	}
 	
 	while (bytes < 2) {
-		bytes = recv(sockfd, wire, 2, 0);
+		bytes = recv(sockfd, (void*)wire, 2, 0);
 		if (bytes == -1 || bytes == 0) {
 			*size = 0;
 			LDNS_FREE(wire);
@@ -420,7 +578,8 @@ ldns_tcp_read_wire(int sockfd, size_t *size)
 	bytes = 0;
 
 	while (bytes < (ssize_t) wire_size) {
-		bytes += recv(sockfd, wire + bytes, (size_t) (wire_size - bytes), 0);
+		bytes += recv(sockfd, (void*) (wire + bytes), 
+				(size_t) (wire_size - bytes), 0);
 		if (bytes == -1 || bytes == 0) {
 			LDNS_FREE(wire);
 			*size = 0;
@@ -447,8 +606,8 @@ ldns_tcp_send(uint8_t **result,  ldns_buffer *qbin, const struct sockaddr_storag
 	if (sockfd == 0) {
 		return LDNS_STATUS_ERR;
 	}
-	
-	answer = ldns_tcp_read_wire(sockfd, answer_size);
+
+	answer = ldns_tcp_read_wire_timeout(sockfd, answer_size, timeout);
 	close(sockfd);
 
 	if (*answer_size == 0) {
@@ -558,8 +717,9 @@ ldns_axfr_start(ldns_resolver *resolver, ldns_rdf *domain, ldns_rr_class class)
         ldns_pkt *query;
         ldns_buffer *query_wire;
 
-        struct sockaddr_storage *ns;
+        struct sockaddr_storage *ns = NULL;
         size_t ns_len = 0;
+        size_t ns_i;
         ldns_status status;
 
         if (!resolver || ldns_resolver_nameserver_count(resolver) < 1) {
@@ -572,16 +732,25 @@ ldns_axfr_start(ldns_resolver *resolver, ldns_rdf *domain, ldns_rr_class class)
                 return LDNS_STATUS_ADDRESS_ERR;
         }
         /* For AXFR, we have to make the connection ourselves */
-        ns = ldns_rdf2native_sockaddr_storage(resolver->_nameservers[0],
-                        ldns_resolver_port(resolver), &ns_len);
+        /* try all nameservers (which usually would mean v4 fallback if 
+         * @hostname is used */
+        for (ns_i = 0;
+             ns_i < ldns_resolver_nameserver_count(resolver) &&
+             resolver->_socket == 0;
+             ns_i++) {
+	        ns = ldns_rdf2native_sockaddr_storage(
+	        	resolver->_nameservers[ns_i],
+			ldns_resolver_port(resolver), &ns_len);
+	
+		resolver->_socket = ldns_tcp_connect(ns, (socklen_t)ns_len, 
+				ldns_resolver_timeout(resolver));
+	}
 
-        resolver->_socket = ldns_tcp_connect(ns, (socklen_t)ns_len, 
-			ldns_resolver_timeout(resolver));
-        if (resolver->_socket == 0) {
-                ldns_pkt_free(query);
-                LDNS_FREE(ns);
-                return LDNS_STATUS_NETWORK_ERR;
-        }
+	if (resolver->_socket == 0) {
+		ldns_pkt_free(query);
+		LDNS_FREE(ns);
+		return LDNS_STATUS_NETWORK_ERR;
+	}
 
 #ifdef HAVE_SSL
 	if (ldns_resolver_tsig_keyname(resolver) && ldns_resolver_tsig_keydata(resolver)) {
