@@ -238,7 +238,7 @@ ldns_pkt_set_edns_do(ldns_pkt *packet, bool value)
 	if (value) {
 		packet->_edns_z = packet->_edns_z | LDNS_EDNS_MASK_DO_BIT;
 	} else {
-		packet->_edns_z = packet->_edns_z & !LDNS_EDNS_MASK_DO_BIT;
+		packet->_edns_z = packet->_edns_z & ~LDNS_EDNS_MASK_DO_BIT;
 	}
 }
 
@@ -793,6 +793,7 @@ ldns_pkt_free(ldns_pkt *packet)
 		ldns_rr_list_deep_free(packet->_authority);
 		ldns_rr_list_deep_free(packet->_additional);
 		ldns_rr_free(packet->_tsig_rr);
+		ldns_rdf_deep_free(packet->_edns_data);
 		LDNS_FREE(packet);
 	}
 }
@@ -860,6 +861,7 @@ ldns_pkt_query_new_frm_str(ldns_pkt **p, const char *name, ldns_rr_type rr_type,
 		ldns_rr_set_owner(question_rr, name_rdf);
 		ldns_rr_set_type(question_rr, rr_type);
 		ldns_rr_set_class(question_rr, rr_class);
+                ldns_rr_set_question(question_rr, true);
 		
 		ldns_pkt_push_rr(packet, LDNS_SECTION_QUESTION, question_rr);
 	} else {
@@ -910,6 +912,7 @@ ldns_pkt_query_new(ldns_rdf *rr_name, ldns_rr_type rr_type, ldns_rr_class rr_cla
 	ldns_rr_set_owner(question_rr, rr_name);
 	ldns_rr_set_type(question_rr, rr_type);
 	ldns_rr_set_class(question_rr, rr_class);
+        ldns_rr_set_question(question_rr, true);
 	
 	packet->_tsig_rr = NULL;
 	
