@@ -65,20 +65,20 @@ usage()
 	printf("  stats				print statistics\n");
 	printf("  stats_noreset			peek at statistics\n");
 	printf("  status			display status of server\n");
-	printf("  verbosity [number]		change logging detail\n");
-	printf("  local_zone [name] [type] 	add new local zone\n");
-	printf("  local_zone_remove [name]	remove local zone and its contents\n");
-	printf("  local_data [RR data...]	add local data, for example\n");
+	printf("  verbosity <number>		change logging detail\n");
+	printf("  local_zone <name> <type>	add new local zone\n");
+	printf("  local_zone_remove <name>	remove local zone and its contents\n");
+	printf("  local_data <RR data...>	add local data, for example\n");
 	printf("				local_data www.example.com A 192.0.2.1\n");
-	printf("  local_data_remove [name]	remove local RR data from name\n");
+	printf("  local_data_remove <name>	remove local RR data from name\n");
 	printf("  dump_cache			print cache to stdout\n");
 	printf("  load_cache			load cache from stdin\n");
-	printf("  lookup [name]			print nameservers for name\n");
-	printf("  flush [name]			flushes common types for name from cache\n");
+	printf("  lookup <name>			print nameservers for name\n");
+	printf("  flush <name>			flushes common types for name from cache\n");
 	printf("  				types:  A, AAAA, MX, PTR, NS,\n");
 	printf("					SOA, CNAME, DNAME, SRV, NAPTR\n");
-	printf("  flush_type [name] [type]	flush name, type from cache\n");
-	printf("  flush_zone [name]		flush everything at or under name\n");
+	printf("  flush_type <name> <type>	flush name, type from cache\n");
+	printf("  flush_zone <name>		flush everything at or under name\n");
 	printf("  				from rr and dnssec caches\n");
 	printf("  flush_stats 			flush statistics, make zero\n");
 	printf("  flush_requestlist 		drop queries that are worked on\n");
