@@ -24,7 +24,7 @@
 
 #include <errno.h>
 #ifdef HAVE_NETDB_H
-#include <netdb.h> 
+#include <netdb.h>
 #endif
 
 #include <limits.h>
@@ -35,12 +35,12 @@
 ldns_status
 ldns_str2rdf_int16(ldns_rdf **rd, const char *shortstr)
 {
-	char *end = NULL;    
+	char *end = NULL;
 	uint16_t *r;
 	r = LDNS_MALLOC(uint16_t);
-	
-	*r = htons((uint16_t)strtol((char *)shortstr, &end, 0));
-	
+
+	*r = htons((uint16_t)strtol((char *)shortstr, &end, 10));
+
 	if(*end != 0) {
 		LDNS_FREE(r);
 		return LDNS_STATUS_INVALID_INT;
@@ -81,7 +81,7 @@ ldns_str2rdf_time(ldns_rdf **rd, const char *time)
 		if (tm.tm_mday < 1 || tm.tm_mday > 31) {
 			goto bad_format;
 		}
-		
+
 		if (tm.tm_hour < 0 || tm.tm_hour > 23) {
 			goto bad_format;
 		}
@@ -102,7 +102,7 @@ ldns_str2rdf_time(ldns_rdf **rd, const char *time)
 		return LDNS_STATUS_OK;
 	} else {
 		/* handle it as 32 bits timestamp */
-		l = htonl((uint32_t)strtol((char*)time, &end, 0));
+		l = htonl((uint32_t)strtol((char*)time, &end, 10));
 		if(*end != 0) {
 			LDNS_FREE(r);
 			return LDNS_STATUS_ERR;
@@ -114,7 +114,7 @@ ldns_str2rdf_time(ldns_rdf **rd, const char *time)
 			return LDNS_STATUS_OK;
 		}
 	}
-	
+
 	bad_format:
 	LDNS_FREE(r);
 	return LDNS_STATUS_INVALID_TIME;
@@ -124,21 +124,25 @@ ldns_status
 ldns_str2rdf_nsec3_salt(ldns_rdf **rd, const char *salt_str)
 {
 	uint8_t salt_length;
-	uint8_t c;
+	int c;
+	int salt_length_str;
 
 	uint8_t *salt;
 	uint8_t *data;
 
-	salt_length = (uint8_t) strlen(salt_str);
-	if (salt_length == 1 && salt_str[0] == '-') {
-		salt_length = 0;
-	} else if (salt_length % 2 != 0) {
+	salt_length_str = strlen(salt_str);
+	if (salt_length_str == 1 && salt_str[0] == '-') {
+		salt_length_str = 0;
+	} else if (salt_length_str % 2 != 0) {
 		return LDNS_STATUS_INVALID_HEX;
 	}
-	
-	salt = LDNS_XMALLOC(uint8_t, salt_length / 2);
-	for (c = 0; c < salt_length; c += 2) {
-		if (isxdigit(salt_str[c]) && isxdigit(salt_str[c+1])) {
+	if (salt_length_str > 512) {
+		return LDNS_STATUS_INVALID_HEX;
+	}
+
+	salt = LDNS_XMALLOC(uint8_t, salt_length_str / 2);
+	for (c = 0; c < salt_length_str; c += 2) {
+		if (isxdigit((int) salt_str[c]) && isxdigit((int) salt_str[c+1])) {
 			salt[c/2] = (uint8_t) ldns_hexdigit_to_int(salt_str[c]) * 16 +
 					  ldns_hexdigit_to_int(salt_str[c+1]);
 		} else {
@@ -146,8 +150,8 @@ ldns_str2rdf_nsec3_salt(ldns_rdf **rd, const char *salt_str)
 			return LDNS_STATUS_INVALID_HEX;
 		}
 	}
-	salt_length = salt_length / 2;
-	
+	salt_length = (uint8_t) (salt_length_str / 2);
+
 	data = LDNS_XMALLOC(uint8_t, 1 + salt_length);
 	data[0] = salt_length;
 	memcpy(&data[1], salt, salt_length);
@@ -177,20 +181,28 @@ ldns_str2rdf_period(ldns_rdf **rd,const char *period)
 	return LDNS_STATUS_OK;
 }
 
-ldns_status 
+ldns_status
 ldns_str2rdf_int32(ldns_rdf **rd, const char *longstr)
 {
-	char *end;  
+	char *end;
 	uint16_t *r = NULL;
 	uint32_t l;
 
 	r = (uint16_t*)LDNS_MALLOC(uint32_t);
-	l = htonl((uint32_t)strtol((char*)longstr, &end, 0));
+	errno = 0; /* must set to zero before call,
+			note race condition on errno */
+	if(*longstr == '-')
+		l = htonl((uint32_t)strtol((char*)longstr, &end, 10));
+	else	l = htonl((uint32_t)strtoul((char*)longstr, &end, 10));
 
 	if(*end != 0) {
 		LDNS_FREE(r);
 		return LDNS_STATUS_ERR;
-        } else {
+     } else {
+		if (errno == ERANGE) {
+			LDNS_FREE(r);
+			return LDNS_STATUS_SYNTAX_INTEGER_OVERFLOW;
+		}
 		memcpy(r, &l, sizeof(uint32_t));
 		*rd = ldns_rdf_new_frm_data(
 			LDNS_RDF_TYPE_INT32, sizeof(uint32_t), r);
@@ -202,12 +214,12 @@ ldns_str2rdf_int32(ldns_rdf **rd, const char *longstr)
 ldns_status
 ldns_str2rdf_int8(ldns_rdf **rd, const char *bytestr)
 {
-	char *end;     
+	char *end;
 	uint8_t *r = NULL;
 
 	r = LDNS_MALLOC(uint8_t);
- 
-	*r = (uint8_t)strtol((char*)bytestr, &end, 0);
+
+	*r = (uint8_t)strtol((char*)bytestr, &end, 10);
 
         if(*end != 0) {
 		LDNS_FREE(r);
@@ -220,6 +232,43 @@ ldns_str2rdf_int8(ldns_rdf **rd, const char *bytestr)
         }
 }
 
+
+/*
+ * Checks whether the escaped value at **s is an octal value or
+ * a 'normally' escaped character (and not eos)
+ *
+ * The string pointer at *s is increased by either 0 (on error), 1 (on
+ * normal escapes), or 3 (on octals)
+ *
+ * Returns the number of bytes read from the escaped string, or
+ * 0 on error
+ */
+static int
+parse_escape(uint8_t *s, uint8_t *q) {
+	uint8_t val;
+	if (strlen((char *)s) > 3 &&
+	    isdigit((int) s[1]) &&
+	    isdigit((int) s[2]) &&
+	    isdigit((int) s[3])) {
+		/* cast this so it fits */
+		val = (uint8_t) ldns_hexdigit_to_int((char) s[1]) * 100 +
+		                ldns_hexdigit_to_int((char) s[2]) * 10 +
+		                ldns_hexdigit_to_int((char) s[3]);
+		*q = val;
+		return 3;
+	} else {
+		s++;
+		if (*s == '\0') {
+			/* apparently the string terminator
+			 * has been escaped...
+		         */
+			return 0;
+		}
+		*q = *s;
+		return 1;
+	}
+}
+
 /*
  * No special care is taken, all dots are translated into
  * label seperators.
@@ -230,10 +279,11 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 {
 	size_t len;
 
-	uint8_t *s,*p,*q, *pq, val, label_len;
+	int esc;
+	uint8_t *s,*p,*q, *pq, label_len;
 	uint8_t buf[LDNS_MAX_DOMAINLEN + 1];
 	*d = NULL;
-	
+
 	len = strlen((char*)str);
 	/* octet representation can make strings a lot longer than actual length */
 	if (len > LDNS_MAX_DOMAINLEN * 4) {
@@ -241,11 +291,11 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 	}
 	if (0 == len) {
 		return LDNS_STATUS_DOMAINNAME_UNDERFLOW;
-	} 
-	
+	}
+
 	/* root label */
 	if (1 == len && *str == '.') {
-		*d = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_DNAME, 1, "\0"); 
+		*d = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_DNAME, 1, "\0");
 		return LDNS_STATUS_OK;
 	}
 
@@ -262,7 +312,7 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 	for (s = p = (uint8_t *) str; *s; s++, q++) {
 		if (q > buf + LDNS_MAX_DOMAINLEN) {
 			return LDNS_STATUS_DOMAINNAME_OVERFLOW;
-		}	
+		}
 		*q = 0;
 		switch (*s) {
 		case '.':
@@ -280,21 +330,13 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 			break;
 		case '\\':
 			/* octet value or literal char */
-			if (strlen((char *)s) > 3 &&
-			    isdigit((int) s[1]) &&
-			    isdigit((int) s[2]) &&
-			    isdigit((int) s[3])) {
-				/* cast this so it fits */
-				val = (uint8_t) ldns_hexdigit_to_int((char) s[1]) * 100 +
-				                ldns_hexdigit_to_int((char) s[2]) * 10 +
-				                ldns_hexdigit_to_int((char) s[3]);
-				*q = val;
-				s += 3;
+			esc = parse_escape(s, q);
+			if (esc > 0) {
+				s += esc;
+				label_len++;
 			} else {
-				s++;
-				*q = *s;
+				return LDNS_STATUS_SYNTAX_BAD_ESCAPE;
 			}
-			label_len++;
 			break;
 		default:
 			*q = *s;
@@ -304,13 +346,16 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 
 	/* add root label if last char was not '.' */
 	if (!ldns_dname_str_absolute(str)) {
+		if (q > buf + LDNS_MAX_DOMAINLEN) {
+			return LDNS_STATUS_DOMAINNAME_OVERFLOW;
+		}
 		len += label_len + 1;
 		*pq = label_len;
 		*q = 0;
 	}
 	len++;
 
-	*d = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_DNAME, len, buf); 
+	*d = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_DNAME, len, buf);
 	return LDNS_STATUS_OK;
 }
 
@@ -345,9 +390,8 @@ ldns_status
 ldns_str2rdf_str(ldns_rdf **rd, const char *str)
 {
 	uint8_t *data;
-	uint8_t val;
 	size_t i, str_i;
-	
+
 	if (strlen(str) > 255) {
 		return LDNS_STATUS_INVALID_STR;
 	}
@@ -356,25 +400,12 @@ ldns_str2rdf_str(ldns_rdf **rd, const char *str)
 	i = 1;
 	for (str_i = 0; str_i < strlen(str); str_i++) {
 		if (str[str_i] == '\\') {
-			if(str_i + 3 < strlen(str) && 
-			   isdigit(str[str_i + 1]) &&
-			   isdigit(str[str_i + 2]) &&
-			   isdigit(str[str_i + 3])) {
-				val = (uint8_t) ldns_hexdigit_to_int((char) str[str_i + 1]) * 100 +
-				                ldns_hexdigit_to_int((char) str[str_i + 2]) * 10 +
-				                ldns_hexdigit_to_int((char) str[str_i + 3]);
-				data[i] = val;
-				i++;
-				str_i += 3;
-			} else {
-				str_i++;
-				data[i] = (uint8_t) str[str_i];
-				i++;
-			}
+			/* octet value or literal char */
+			str_i += (size_t) parse_escape((uint8_t*) &str[str_i], (uint8_t*) &data[i]);
 		} else {
 			data[i] = (uint8_t) str[str_i];
-			i++;
 		}
+		i++;
 	}
 	data[0] = i - 1;
 	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_STR, i, data);
@@ -397,7 +428,7 @@ ldns_str2rdf_apl(ldns_rdf **rd, const char *str)
 	uint8_t prefix;
 
 	uint8_t *data;
-	
+
 	size_t i = 0;
 
 	/* [!]afi:address/prefix */
@@ -413,7 +444,7 @@ ldns_str2rdf_apl(ldns_rdf **rd, const char *str)
 	}
 
 	family = (uint16_t) atoi(my_str);
-	
+
 	my_str = strchr(my_str, ':') + 1;
 
 	/* need ip addr and only ip addr for inet_pton */
@@ -461,7 +492,7 @@ ldns_str2rdf_apl(ldns_rdf **rd, const char *str)
 		/* set bit 1 of byte 3 */
 		data[3] = data[3] | 0x80;
 	}
-	
+
 	memcpy(data + 4, afdpart, afdlength);
 
 	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_APL, afdlength + 4, data);
@@ -477,11 +508,11 @@ ldns_str2rdf_b64(ldns_rdf **rd, const char *str)
 {
 	uint8_t *buffer;
 	int16_t i;
-	
-	buffer = LDNS_XMALLOC(uint8_t, b64_ntop_calculate_size(strlen(str)));
-	
-	i = (uint16_t)b64_pton((const char*)str, buffer, 
-	                        b64_ntop_calculate_size(strlen(str)));
+
+	buffer = LDNS_XMALLOC(uint8_t, ldns_b64_ntop_calculate_size(strlen(str)));
+
+	i = (uint16_t)ldns_b64_pton((const char*)str, buffer,
+						   ldns_b64_ntop_calculate_size(strlen(str)));
 	if (-1 == i) {
 		LDNS_FREE(buffer);
 		return LDNS_STATUS_INVALID_B64;
@@ -500,12 +531,12 @@ ldns_str2rdf_b32_ext(ldns_rdf **rd, const char *str)
 	uint8_t *buffer;
 	int i;
 	/* first byte contains length of actual b32 data */
-	uint8_t len = b32_pton_calculate_size(strlen(str));
+	uint8_t len = ldns_b32_pton_calculate_size(strlen(str));
 	buffer = LDNS_XMALLOC(uint8_t, len + 1);
 	buffer[0] = len;
-	
-	i = b32_pton_extended_hex((const char*)str, strlen(str), buffer + 1, 
-                                  b32_ntop_calculate_size(strlen(str)));
+
+	i = ldns_b32_pton_extended_hex((const char*)str, strlen(str), buffer + 1,
+							 ldns_b32_ntop_calculate_size(strlen(str)));
 	if (i < 0) {
 		return LDNS_STATUS_INVALID_B32_EXT;
 	} else {
@@ -520,38 +551,43 @@ ldns_str2rdf_b32_ext(ldns_rdf **rd, const char *str)
 ldns_status
 ldns_str2rdf_hex(ldns_rdf **rd, const char *str)
 {
-        uint8_t *t, *t_orig;
-        int i;
+	uint8_t *t, *t_orig;
+	int i;
 	size_t len;
 
 	len = strlen(str);
 
-        if (len % 2 != 0) {
-                return LDNS_STATUS_INVALID_HEX;
-        } else if (len > LDNS_MAX_RDFLEN * 2) {
+	if (len > LDNS_MAX_RDFLEN * 2) {
 		return LDNS_STATUS_LABEL_OVERFLOW;
-        } else {
-		t = LDNS_XMALLOC(uint8_t, (len / 2));
+	} else {
+		t = LDNS_XMALLOC(uint8_t, (len / 2) + 1);
 		t_orig = t;
-                /* Now process octet by octet... */
-                while (*str) {
-                        *t = 0;
-                        for (i = 16; i >= 1; i -= 15) {
-                                if (isxdigit(*str)) {
-                                        *t += ldns_hexdigit_to_int(*str) * i;
-                                } else {
-					/* error or be lenient and skip? */
-                                        /*return LDNS_STATUS_ERR;*/
-                                }
-                                ++str;
-                        }
-                        ++t;
-                }
-                t = t_orig;
-		*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX, len / 2, t);
-		LDNS_FREE(t);
-        }
-        return LDNS_STATUS_OK;
+		/* Now process octet by octet... */
+		while (*str) {
+			*t = 0;
+			if (isspace((int) *str)) {
+				str++;
+			} else {
+				for (i = 16; i >= 1; i -= 15) {
+					while (*str && isspace((int) *str)) { str++; }
+					if (*str) {
+						if (isxdigit((int) *str)) {
+							*t += ldns_hexdigit_to_int(*str) * i;
+						} else {
+							return LDNS_STATUS_ERR;
+						}
+						++str;
+					}
+				}
+				++t;
+			}
+		}
+		*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX,
+		                            (size_t) (t - t_orig),
+		                            t_orig);
+		LDNS_FREE(t_orig);
+	}
+	return LDNS_STATUS_OK;
 }
 
 ldns_status
@@ -559,72 +595,27 @@ ldns_str2rdf_nsec(ldns_rdf **rd, const char *str)
 {
 	const char *delimiters = "\n\t ";
 	char *token = LDNS_XMALLOC(char, LDNS_MAX_RDFLEN);
-	uint8_t *bitmap = LDNS_XMALLOC(uint8_t, 1);
-	uint16_t bm_len = 0;
 	ldns_buffer *str_buf;
 	ssize_t c;
 	uint16_t cur_type;
-	uint8_t cur_data[32];
-	uint8_t cur_window = 0;
-	uint8_t cur_window_max = 0;
-	uint16_t cur_data_size = 0;
-	uint16_t i;
-	uint8_t *data = NULL;
+	size_t type_count = 0;
+	ldns_rr_type type_list[1024];
 
 	str_buf = LDNS_MALLOC(ldns_buffer);
 	ldns_buffer_new_frm_data(str_buf, (char *)str, strlen(str));
 
-	bitmap[0] = 0;
 	while ((c = ldns_bget_token(str_buf, token, delimiters, LDNS_MAX_RDFLEN)) != -1) {
 		cur_type = ldns_get_rr_type_by_name(token);
-		if ((cur_type / 8) + 1 > bm_len) {
-			bitmap = LDNS_XREALLOC(bitmap, uint8_t, (cur_type / 8) + 1);
-			/* set to 0 */
-			for (; bm_len <= cur_type / 8; bm_len++) {
-				bitmap[bm_len] = 0;
-			}
-		}
-		ldns_set_bit(bitmap + (int) cur_type / 8, (int) (7 - (cur_type % 8)), true);
+		type_list[type_count] = cur_type;
+		type_count++;
 	}
 
-	memset(cur_data, 0, 32);
-	for (i = 0; i < bm_len; i++) {
-		if (i / 32 > cur_window) {
-			/* check, copy, new */
-			if (cur_window_max > 0) {
-				/* this window has stuff, add it */
-				data = LDNS_XREALLOC(data, uint8_t, cur_data_size + cur_window_max + 3);
-				data[cur_data_size] = cur_window;
-				data[cur_data_size + 1] = cur_window_max + 1;
-				memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
-				cur_data_size += cur_window_max + 3;
-			}
-			cur_window++;
-			cur_window_max = 0;
-			memset(cur_data, 0, 32);
-		} else {
-			cur_data[i%32] = bitmap[i];
-			if (bitmap[i] > 0) {
-				cur_window_max = i%32;
-			}
-		}
-	}
-	if (cur_window_max > 0) {
-		/* this window has stuff, add it */
-		data = LDNS_XREALLOC(data, uint8_t, cur_data_size + cur_window_max + 3);
-		data[cur_data_size] = cur_window;
-		data[cur_data_size + 1] = cur_window_max + 1;
-		memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
-		cur_data_size += cur_window_max + 3;
-	}
+	*rd = ldns_dnssec_create_nsec_bitmap(type_list,
+	                                     type_count,
+	                                     LDNS_RR_TYPE_NSEC);
 
-	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC, cur_data_size, data);
-	if(data)
-		LDNS_FREE(data);
 	if (token)
 		LDNS_FREE(token);
-	if(bitmap)
-		LDNS_FREE(bitmap);
 	ldns_buffer_free(str_buf);
 	return LDNS_STATUS_OK;
 }
@@ -681,7 +672,7 @@ ldns_str2rdf_cert_alg(ldns_rdf **rd, const char *str)
 
 	return st;
 }
-		
+
 /* An alg field can either be specified as a 8 bits number
  * or by its symbolic name. Handle both
  */
@@ -706,11 +697,11 @@ ldns_str2rdf_alg(ldns_rdf **rd, const char *str)
 	}
 	return st;
 }
-		
+
 ldns_status
 ldns_str2rdf_unknown(ldns_rdf **rd, const char *str)
 {
-	/* this should be caught in an earlier time (general str2host for 
+	/* this should be caught in an earlier time (general str2host for
 	   rr's */
 	rd = rd;
 	str = str;
@@ -761,7 +752,7 @@ loc_parse_cm(char* my_str, char** endstr, uint8_t* m, uint8_t* e)
 		val /= 10;
 	}
 	*m = (uint8_t)val;
-	
+
 	if (*e > 9)
 		return 0;
 	if (*my_str == 'm' || *my_str == 'M') {
@@ -786,7 +777,7 @@ ldns_str2rdf_loc(ldns_rdf **rd, const char *str)
 	uint8_t size_b = 1, size_e = 2;
 	uint8_t horiz_pre_b = 1, horiz_pre_e = 6;
 	uint8_t vert_pre_b = 1, vert_pre_e = 3;
-	
+
 	double s = 0.0;
 	bool northerness;
 	bool easterness;
@@ -794,17 +785,17 @@ ldns_str2rdf_loc(ldns_rdf **rd, const char *str)
 	char *my_str = (char *) str;
 
 	/* only support version 0 */
-	if (isdigit(*my_str)) {
+	if (isdigit((int) *my_str)) {
 		h = (uint32_t) strtol(my_str, &my_str, 10);
 	} else {
 		return LDNS_STATUS_INVALID_STR;
 	}
 
-	while (isblank(*my_str)) {
+	while (isblank((int) *my_str)) {
 		my_str++;
 	}
 
-	if (isdigit(*my_str)) {
+	if (isdigit((int) *my_str)) {
 		m = (uint32_t) strtol(my_str, &my_str, 10);
 	} else if (*my_str == 'N' || *my_str == 'S') {
 		goto north;
@@ -812,15 +803,15 @@ ldns_str2rdf_loc(ldns_rdf **rd, const char *str)
 		return LDNS_STATUS_INVALID_STR;
 	}
 
-	while (isblank(*my_str)) {
+	while (isblank((int) *my_str)) {
 		my_str++;
 	}
 
-	if (isdigit(*my_str)) {
+	if (isdigit((int) *my_str)) {
 		s = strtod(my_str, &my_str);
 	}
 north:
-	while (isblank(*my_str)) {
+	while (isblank((int) *my_str)) {
 		my_str++;
 	}
 
@@ -850,17 +841,17 @@ north:
 		my_str++;
 	}
 
-	if (isdigit(*my_str)) {
+	if (isdigit((int) *my_str)) {
 		h = (uint32_t) strtol(my_str, &my_str, 10);
 	} else {
 		return LDNS_STATUS_INVALID_STR;
 	}
 
-	while (isblank(*my_str)) {
+	while (isblank((int) *my_str)) {
 		my_str++;
 	}
 
-	if (isdigit(*my_str)) {
+	if (isdigit((int) *my_str)) {
 		m = (uint32_t) strtol(my_str, &my_str, 10);
 	} else if (*my_str == 'E' || *my_str == 'W') {
 		goto east;
@@ -872,7 +863,7 @@ north:
 		my_str++;
 	}
 
-	if (isdigit(*my_str)) {
+	if (isdigit((int) *my_str)) {
 		s = strtod(my_str, &my_str);
 	}
 
@@ -949,16 +940,16 @@ ldns_str2rdf_wks(ldns_rdf **rd, const char *str)
 	uint8_t *bitmap = NULL;
 	uint8_t *data;
 	int bm_len = 0;
-	
+
 	struct protoent *proto = NULL;
 	struct servent *serv = NULL;
 	int serv_port;
-	
+
 	ldns_buffer *str_buf;
-	
+
 	char *proto_str = NULL;
 	char *token = LDNS_XMALLOC(char, 50);
-	
+
 	str_buf = LDNS_MALLOC(ldns_buffer);
 	ldns_buffer_new_frm_data(str_buf, (char *)str, strlen(str));
 
@@ -977,7 +968,7 @@ ldns_str2rdf_wks(ldns_rdf **rd, const char *str)
 			} else {
 				serv_port = atoi(token);
 			}
-			if (serv_port / 8 > bm_len) {
+			if (serv_port / 8 >= bm_len) {
 				bitmap = LDNS_XREALLOC(bitmap, uint8_t, (serv_port / 8) + 1);
 				/* set to zero to be sure */
 				for (; bm_len <= serv_port / 8; bm_len++) {
@@ -987,22 +978,31 @@ ldns_str2rdf_wks(ldns_rdf **rd, const char *str)
 			ldns_set_bit(bitmap + (serv_port / 8), 7 - (serv_port % 8), true);
 		}
 	}
-	
+
+	if (!proto_str) {
+		LDNS_FREE(token);
+		LDNS_FREE(str_buf);
+		return LDNS_STATUS_INVALID_STR;
+	}
+
 	data = LDNS_XMALLOC(uint8_t, bm_len + 1);
-	proto = getprotobyname(proto_str);
+    if (proto_str)
+		proto = getprotobyname(proto_str);
 	if (proto) {
 		data[0] = (uint8_t) proto->p_proto;
-	} else {
+	} else if (proto_str) {
 		data[0] = (uint8_t) atoi(proto_str);
+	} else {
+		data[0] = 0;
 	}
 	memcpy(data + 1, bitmap, (size_t) bm_len);
-	
+
 	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_WKS, (uint16_t) (bm_len + 1), data);
 
+	LDNS_FREE(data);
 	LDNS_FREE(token);
 	ldns_buffer_free(str_buf);
 	LDNS_FREE(bitmap);
-	LDNS_FREE(data);
 	free(proto_str);
 #ifdef HAVE_ENDSERVENT
 	endservent();
@@ -1010,17 +1010,173 @@ ldns_str2rdf_wks(ldns_rdf **rd, const char *str)
 #ifdef HAVE_ENDPROTOENT
 	endprotoent();
 #endif
-	
+
 	return LDNS_STATUS_OK;
 }
 
 ldns_status
 ldns_str2rdf_nsap(ldns_rdf **rd, const char *str)
 {
+    size_t len, i;
+    char* nsap_str = (char*) str;
+
 	/* just a hex string with optional dots? */
 	if (str[0] != '0' || str[1] != 'x') {
 		return LDNS_STATUS_INVALID_STR;
 	} else {
+		len = strlen(str);
+		for (i=0; i < len; i++) {
+			if (nsap_str[i] == '.')
+				nsap_str[i] = ' ';
+        }
 		return ldns_str2rdf_hex(rd, str+2);
 	}
 }
+
+ldns_status
+ldns_str2rdf_atma(ldns_rdf **rd, const char *str)
+{
+    size_t len, i;
+    char* atma_str = (char*) str;
+	ldns_status status;
+
+	/* just a hex string with optional dots? */
+	len = strlen(str);
+	for (i=0; i < len; i++) {
+		if (atma_str[i] == '.')
+			atma_str[i] = ' ';
+	}
+	status = ldns_str2rdf_hex(rd, str);
+    if (status != LDNS_STATUS_OK) {
+		; /* probably in e.164 format than */
+	}
+	return status;
+}
+
+ldns_status
+ldns_str2rdf_ipseckey(ldns_rdf **rd, const char *str)
+{
+	uint8_t precedence = 0;
+	uint8_t gateway_type = 0;
+	uint8_t algorithm = 0;
+	char* gateway = NULL;
+	char* publickey = NULL;
+	uint8_t *data;
+	ldns_buffer *str_buf;
+	char *token = LDNS_XMALLOC(char, 256);
+	int token_count = 0;
+	int ipseckey_len = 0;
+	ldns_rdf* gateway_rdf = NULL;
+	ldns_rdf* publickey_rdf = NULL;
+	ldns_status status = LDNS_STATUS_OK;
+
+	str_buf = LDNS_MALLOC(ldns_buffer);
+	ldns_buffer_new_frm_data(str_buf, (char *)str, strlen(str));
+	while(ldns_bget_token(str_buf, token, "\t\n ", strlen(str)) > 0) {
+		switch (token_count) {
+				case 0:
+					precedence = atoi(token);
+					break;
+				case 1:
+					gateway_type = atoi(token);
+					break;
+				case 2:
+					algorithm = atoi(token);
+					break;
+				case 3:
+					gateway = strdup(token);
+					if (!gateway || (gateway_type == 0 &&
+							(token[0] != '.' || token[1] != '\0'))) {
+						LDNS_FREE(gateway);
+						LDNS_FREE(token);
+						LDNS_FREE(str_buf);
+						return LDNS_STATUS_INVALID_STR;
+					}
+					break;
+				case 4:
+					publickey = strdup(token);
+					break;
+				default:
+					LDNS_FREE(token);
+					LDNS_FREE(str_buf);
+					return LDNS_STATUS_INVALID_STR;
+					break;
+		}
+		token_count++;
+	}
+
+	if (!gateway || !publickey) {
+		if (gateway)
+			LDNS_FREE(gateway);
+		if (publickey)
+			LDNS_FREE(publickey);
+		LDNS_FREE(token);
+		LDNS_FREE(str_buf);
+		return LDNS_STATUS_INVALID_STR;
+	}
+
+	if (gateway_type == 1) {
+		status = ldns_str2rdf_a(&gateway_rdf, gateway);
+	} else if (gateway_type == 2) {
+		status = ldns_str2rdf_aaaa(&gateway_rdf, gateway);
+	} else if (gateway_type == 3) {
+		status = ldns_str2rdf_dname(&gateway_rdf, gateway);
+	}
+
+	if (status != LDNS_STATUS_OK) {
+		if (gateway)
+			LDNS_FREE(gateway);
+		if (publickey)
+			LDNS_FREE(publickey);
+		LDNS_FREE(token);
+		LDNS_FREE(str_buf);
+		return LDNS_STATUS_INVALID_STR;
+	}
+
+	status = ldns_str2rdf_b64(&publickey_rdf, publickey);
+
+	if (status != LDNS_STATUS_OK) {
+		if (gateway)
+			LDNS_FREE(gateway);
+		if (publickey)
+			LDNS_FREE(publickey);
+		LDNS_FREE(token);
+		LDNS_FREE(str_buf);
+		return LDNS_STATUS_INVALID_STR;
+	}
+
+	/* now copy all into one ipseckey rdf */
+	if (gateway_type)
+		ipseckey_len = 3 + ldns_rdf_size(gateway_rdf) + ldns_rdf_size(publickey_rdf);
+	else
+		ipseckey_len = 3 + ldns_rdf_size(publickey_rdf);
+
+	data = LDNS_XMALLOC(uint8_t, ipseckey_len);
+
+	data[0] = precedence;
+	data[1] = gateway_type;
+	data[2] = algorithm;
+
+	if (gateway_type) {
+		memcpy(data + 3,
+			ldns_rdf_data(gateway_rdf), ldns_rdf_size(gateway_rdf));
+		memcpy(data + 3 + ldns_rdf_size(gateway_rdf),
+			ldns_rdf_data(publickey_rdf), ldns_rdf_size(publickey_rdf));
+	} else {
+		memcpy(data + 3,
+			ldns_rdf_data(publickey_rdf), ldns_rdf_size(publickey_rdf));
+	}
+
+	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_IPSECKEY, (uint16_t) ipseckey_len, data);
+
+	if (gateway)
+		LDNS_FREE(gateway);
+	if (publickey)
+		LDNS_FREE(publickey);
+	LDNS_FREE(token);
+	ldns_buffer_free(str_buf);
+	ldns_rdf_free(gateway_rdf);
+	ldns_rdf_free(publickey_rdf);
+	LDNS_FREE(data);
+	return LDNS_STATUS_OK;
+}
