@@ -42,6 +42,7 @@
 
 #include "config.h"
 #include "libunbound/unbound.h"
+#include "libunbound/context.h"
 #include "util/locks.h"
 #include "util/log.h"
 
@@ -299,9 +300,11 @@ ext_thread(void* arg)
 			if(i > 100) {
 				lock_basic_lock(&async_ids[i-100].lock);
 				r = ub_cancel(inf->ctx, async_ids[i-100].id);
-				async_ids[i-100].cancel=1;
+				if(r != UB_NOID)
+					async_ids[i-100].cancel=1;
 				lock_basic_unlock(&async_ids[i-100].lock);
-				checkerr("ub_cancel", r);
+				if(r != UB_NOID) 
+					checkerr("ub_cancel", r);
 			}
 		} else if(inf->thread_num > NUMTHR/2) {
 			/* async */
@@ -367,6 +370,8 @@ int main(int argc, char** argv)
 	struct lookinfo* lookups;
 	int i, r, cancel=0, blocking=0, ext=0;
 
+	/* init log now because solaris thr_key_create() is not threadsafe */
+	log_init(0,0,0);
 	/* lock debug start (if any) */
 	checklock_start();
 
@@ -467,7 +472,8 @@ int main(int argc, char** argv)
 		for(i=0; i<argc; i++) {
 			fprintf(stderr, "cancel %s\n", argv[i]);
 			r = ub_cancel(ctx, lookups[i].async_id);
-			checkerr("ub_cancel", r);
+			if(r != UB_NOID) 
+				checkerr("ub_cancel", r);
 		}
 		num_wait = 0;
 	}
