@@ -47,6 +47,9 @@
 #include "util/configparser.h"
 #include "util/net_help.h"
 #include "util/data/msgparse.h"
+#include "util/module.h"
+#include "util/regional.h"
+#include "util/data/dname.h"
 /** global config during parsing */
 struct config_parser_state* cfg_parser = 0;
 /** lex in file */
@@ -59,8 +62,6 @@ int ub_c_parse(void);
 int ub_c_lex(void);
 /** wrap function */
 int ub_c_wrap(void);
-/** print error with file and line number */
-void ub_c_error(const char *message);
 
 /** init ports possible for use */
 static void init_outgoing_availports(int* array, int num);
@@ -94,6 +95,7 @@ config_create()
 	cfg->outgoing_num_tcp = 2; /* leaves 64-52=12 for: 4if,1stop,thread4 */
 	cfg->incoming_num_tcp = 2; 
 #endif
+	cfg->edns_buffer_size = 4096; /* 4k from rfc recommendation */
 	cfg->msg_buffer_size = 65552; /* 64 k + a small margin */
 	cfg->msg_cache_size = 4 * 1024 * 1024;
 	cfg->msg_cache_slabs = 4;
@@ -125,6 +127,7 @@ config_create()
 	cfg->root_hints = NULL;
 	cfg->do_daemonize = 1;
 	cfg->if_automatic = 0;
+	cfg->socket_rcvbuf = 0;
 	cfg->num_ifs = 0;
 	cfg->ifs = NULL;
 	cfg->num_out_ifs = 0;
@@ -145,6 +148,7 @@ config_create()
 	cfg->hide_version = 0;
 	cfg->identity = NULL;
 	cfg->version = NULL;
+	cfg->auto_trust_anchor_file_list = NULL;
 	cfg->trust_anchor_file_list = NULL;
 	cfg->trust_anchor_list = NULL;
 	cfg->trusted_keys_file_list = NULL;
@@ -156,7 +160,11 @@ config_create()
 	cfg->val_sig_skew_max = 86400; /* at most timezone settings trouble */
 	cfg->val_clean_additional = 1;
 	cfg->val_log_level = 0;
+	cfg->val_log_squelch = 0;
 	cfg->val_permissive_mode = 0;
+	cfg->add_holddown = 30*24*3600;
+	cfg->del_holddown = 30*24*3600;
+	cfg->keep_missing = 366*24*3600; /* one year plus a little leeway */
 	cfg->key_cache_size = 4 * 1024 * 1024;
 	cfg->key_cache_slabs = 4;
 	cfg->neg_cache_size = 1 * 1024 * 1024;
@@ -207,6 +215,8 @@ struct config_file* config_create_forlib()
 	cfg->neg_cache_size = 100 * 1024;
 	cfg->donotquery_localhost = 0; /* allow, so that you can ask a
 		forward nameserver running on localhost */
+	cfg->val_log_level = 2; /* to fill why_bogus with */
+	cfg->val_log_squelch = 1;
 	return cfg;
 }
 
@@ -268,6 +278,9 @@ int config_set_option(struct config_file* cfg, const char* opt,
 	} else if(strcmp(opt, "incoming-num-tcp:") == 0) {
 		IS_NUMBER_OR_ZERO;
 		cfg->incoming_num_tcp = (size_t)atoi(val);
+	} else if(strcmp(opt, "edns-buffer-size:") == 0) {
+		IS_NONZERO_NUMBER;
+		cfg->edns_buffer_size = (size_t)atoi(val);
 	} else if(strcmp(opt, "msg-buffer-size:") == 0) {
 		IS_NONZERO_NUMBER;
 		cfg->msg_buffer_size = (size_t)atoi(val);
@@ -282,6 +295,8 @@ int config_set_option(struct config_file* cfg, const char* opt,
 	} else if(strcmp(opt, "jostle-timeout:") == 0) {
 		IS_NUMBER_OR_ZERO;
 		cfg->jostle_time = (size_t)atoi(val);
+	} else if(strcmp(opt, "so-rcvbuf:") == 0) {
+		return cfg_parse_memsize(val, &cfg->socket_rcvbuf);
 	} else if(strcmp(opt, "rrset-cache-size:") == 0) {
 		return cfg_parse_memsize(val, &cfg->rrset_cache_size);
 	} else if(strcmp(opt, "rrset-cache-slabs:") == 0) {
@@ -343,6 +358,9 @@ int config_set_option(struct config_file* cfg, const char* opt,
 		cfg->donotquery_localhost = (strcmp(val, "yes") == 0);
 	} else if(strcmp(opt, "do-not-query-address:") == 0) {
 		return cfg_strlist_insert(&cfg->donotqueryaddrs, strdup(val));
+	} else if(strcmp(opt, "auto-trust-anchor-file:") == 0) {
+		return cfg_strlist_insert(&cfg->auto_trust_anchor_file_list, 
+			strdup(val));
 	} else if(strcmp(opt, "trust-anchor-file:") == 0) {
 		return cfg_strlist_insert(&cfg->trust_anchor_file_list, 
 			strdup(val));
@@ -379,12 +397,24 @@ int config_set_option(struct config_file* cfg, const char* opt,
 	} else if(strcmp(opt, "val-log-level:") == 0) {
 		IS_NUMBER_OR_ZERO;
 		cfg->val_log_level = atoi(val);
+	} else if(strcmp(opt, "val-log-squelch:") == 0) {
+		IS_YES_OR_NO;
+		cfg->val_log_squelch = (strcmp(val, "yes") == 0);
 	} else if(strcmp(opt, "val-permissive-mode:") == 0) {
 		IS_YES_OR_NO;
 		cfg->val_permissive_mode = (strcmp(val, "yes") == 0);
 	} else if(strcmp(opt, "val-nsec3-keysize-iterations:") == 0) {
 		free(cfg->val_nsec3_key_iterations);
 		return (cfg->val_nsec3_key_iterations = strdup(val)) != NULL;
+	} else if(strcmp(opt, "add-holddown:") == 0) {
+		IS_NUMBER_OR_ZERO;
+		cfg->add_holddown = (unsigned)atoi(val);
+	} else if(strcmp(opt, "del-holddown:") == 0) {
+		IS_NUMBER_OR_ZERO;
+		cfg->del_holddown = (unsigned)atoi(val);
+	} else if(strcmp(opt, "keep-missing:") == 0) {
+		IS_NUMBER_OR_ZERO;
+		cfg->keep_missing = (unsigned)atoi(val);
 	} else if(strcmp(opt, "key-cache-size:") == 0) {
 		return cfg_parse_memsize(val, &cfg->key_cache_size);
 	} else if(strcmp(opt, "key-cache-slabs:") == 0) {
@@ -539,6 +569,7 @@ config_delete(struct config_file* cfg)
 	free(cfg->outgoing_avail_ports);
 	config_delstrlist(cfg->private_address);
 	config_delstrlist(cfg->private_domain);
+	config_delstrlist(cfg->auto_trust_anchor_file_list);
 	config_delstrlist(cfg->trust_anchor_file_list);
 	config_delstrlist(cfg->trusted_keys_file_list);
 	config_delstrlist(cfg->trust_anchor_list);
@@ -823,6 +854,7 @@ config_apply(struct config_file* config)
 {
 	MAX_TTL = (uint32_t)config->max_ttl;
 	MIN_TTL = (uint32_t)config->min_ttl;
+	EDNS_ADVERTISED_SIZE = (uint16_t)config->edns_buffer_size;
 	log_set_time_asc(config->log_time_ascii);
 }
 
@@ -1077,3 +1109,113 @@ char* cfg_ptr_reverse(char* str)
 	}
 	return result;
 }
+
+void errinf(struct module_qstate* qstate, const char* str)
+{
+	struct config_strlist* p;
+	if(qstate->env->cfg->val_log_level < 2 || !str)
+		return;
+	p = (struct config_strlist*)regional_alloc(qstate->region, sizeof(*p));
+	if(!p) {
+		log_err("malloc failure in validator-error-info string");
+		return;
+	}
+	p->next = NULL;
+	p->str = regional_strdup(qstate->region, str);
+	if(!p->str) {
+		log_err("malloc failure in validator-error-info string");
+		return;
+	}
+	/* add at end */
+	if(qstate->errinf) {
+		struct config_strlist* q = qstate->errinf;
+		while(q->next) 
+			q = q->next;
+		q->next = p;
+	} else	qstate->errinf = p;
+}
+
+void errinf_origin(struct module_qstate* qstate, struct sock_list *origin)
+{
+	struct sock_list* p;
+	if(qstate->env->cfg->val_log_level < 2)
+		return;
+	for(p=origin; p; p=p->next) {
+		char buf[256];
+		if(p == origin)
+			snprintf(buf, sizeof(buf), "from ");
+		else	snprintf(buf, sizeof(buf), "and ");
+		if(p->len == 0)
+			snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), 
+				"cache");
+		else 
+			addr_to_str(&p->addr, p->len, buf+strlen(buf),
+				sizeof(buf)-strlen(buf));
+		errinf(qstate, buf);
+	}
+}
+
+char* errinf_to_str(struct module_qstate* qstate)
+{
+	char buf[20480];
+	char* p = buf;
+	size_t left = sizeof(buf);
+	struct config_strlist* s;
+	char dname[LDNS_MAX_DOMAINLEN+1];
+	char* t = ldns_rr_type2str(qstate->qinfo.qtype);
+	char* c = ldns_rr_class2str(qstate->qinfo.qclass);
+	if(!t || !c) {
+		free(t);
+		free(c);
+		log_err("malloc failure in errinf_to_str");
+		return NULL;
+	}
+	dname_str(qstate->qinfo.qname, dname);
+	snprintf(p, left, "validation failure <%s %s %s>:", dname, t, c);
+	free(t);
+	free(c);
+	left -= strlen(p); p += strlen(p);
+	if(!qstate->errinf)
+		snprintf(p, left, " misc failure");
+	else for(s=qstate->errinf; s; s=s->next) {
+		snprintf(p, left, " %s", s->str);
+		left -= strlen(p); p += strlen(p);
+	}
+	p = strdup(buf);
+	if(!p)
+		log_err("malloc failure in errinf_to_str");
+	return p;
+}
+
+void errinf_rrset(struct module_qstate* qstate, struct ub_packed_rrset_key *rr)
+{
+	char buf[1024];
+	char dname[LDNS_MAX_DOMAINLEN+1];
+	char *t, *c;
+	if(qstate->env->cfg->val_log_level < 2 || !rr)
+		return;
+	t = ldns_rr_type2str(ntohs(rr->rk.type));
+	c = ldns_rr_class2str(ntohs(rr->rk.rrset_class));
+	if(!t || !c) {
+		free(t);
+		free(c);
+		log_err("malloc failure in errinf_rrset");
+		return;
+	}
+	dname_str(qstate->qinfo.qname, dname);
+	snprintf(buf, sizeof(buf), "for <%s %s %s>", dname, t, c);
+	free(t);
+	free(c);
+	errinf(qstate, buf);
+}
+
+void errinf_dname(struct module_qstate* qstate, const char* str, uint8_t* dname)
+{
+	char b[1024];
+	char buf[LDNS_MAX_DOMAINLEN+1];
+	if(qstate->env->cfg->val_log_level < 2 || !str || !dname)
+		return;
+	dname_str(dname, buf);
+	snprintf(b, sizeof(b), "%s %s", str, buf);
+	errinf(qstate, b);
+}
