@@ -61,6 +61,7 @@
 #include "validator/val_sigcrypt.h"
 #include "validator/val_kentry.h"
 #include "validator/val_neg.h"
+#include "validator/autotrust.h"
 #include "util/data/msgreply.h"
 #include "util/data/packed_rrset.h"
 #include "util/storage/slabhash.h"
@@ -103,6 +104,7 @@ fptr_whitelist_comm_timer(void (*fptr)(void*))
 	if(fptr == &pending_udp_timer_cb) return 1;
 	else if(fptr == &outnet_tcptimer) return 1;
 	else if(fptr == &worker_stat_timer_cb) return 1;
+	else if(fptr == &worker_probe_timer_cb) return 1;
 #ifdef UB_ON_WINDOWS
 	else if(fptr == &wsvc_cron_cb) return 1;
 #endif
@@ -181,6 +183,8 @@ fptr_whitelist_rbtree_cmp(int (*fptr) (const void *, const void *))
 	else if(fptr == &context_query_cmp) return 1;
 	else if(fptr == &val_neg_data_compare) return 1;
 	else if(fptr == &val_neg_zone_compare) return 1;
+	else if(fptr == &probetree_cmp) return 1;
+	else if(fptr == &replay_var_compare) return 1;
 	return 0;
 }
 
@@ -381,5 +385,6 @@ int fptr_whitelist_mesh_cb(mesh_cb_func_t fptr)
 {
 	if(fptr == &libworker_fg_done_cb) return 1;
 	else if(fptr == &libworker_bg_done_cb) return 1;
+	else if(fptr == &probe_answer_cb) return 1;
 	return 0;
 }
